import * as React from "react";
import { useEffect, useMemo, useState } from "@theia/core/shared/react";
import { createUseStyles } from "react-jss";
import { EMetricWidgetType, IMetricWidget } from "../../constants";
import SingleValueContainer from "../SingleValueContainer";
import { useMetricWidgetContext } from "../MetricWidgetContext";

const useStyles = createUseStyles({
  innerContainer: {
    display: "flex",
    flexDirection: "column",
    padding: 24,
    width: "100%",
    justifyContent: "space-between",
  },
  outerContainer: {
    display: "flex",
    height: "128px",
    backgroundColor: "var(--theia-button-background)",
    color: "var(--theia-button-foreground)",
    "&:hover": {
      backgroundColor: "var(--theia-button-hoverBackground)",
    },
    overflow: "auto",
    borderRadius: "6px",
    cursor: "pointer",
  },
  label: {
    margin: 0,
    fontWeight: 600,
    color: "var(--theia-descriptionForeground)",
    fontSize: 18,
  },
  value: {
    margin: "0 0 0 auto",
    fontWeight: 600,
  },
  description: {
    margin: 0,
    color: "var(--theia-descriptionForeground)",
    fontSize: 16,
    overflow: "auto",
  },
  descriptionContainer: {
    padding: 24,
    width: "100%",
  },
});
interface IMetricWidgetProps {
  widget: IMetricWidget;
}
const MetricWidget = (props: IMetricWidgetProps) => {
  const { widget } = props;
  const {
    value_key,
    metric_url,
    key_separator,
    response_format,
    unit,
    fallback_value,
  } = widget;
  const classes = useStyles();
  const [flipped, setFlipped] = useState<boolean>(false);
  const {
    fetchMetricUrl,
    getValueByKey,
    responseDict,
    globalResponse,
    globalResponseFormat,
  } = useMetricWidgetContext();

  useEffect(() => {
    fetchMetricUrl(widget.metric_url, widget.response_format);
  }, []);

  if (widget.type === EMetricWidgetType.SINGLE_VALUE_CONTAINER) {
    return (
      <SingleValueContainer widget={widget}>
        {widget.widgets?.map((widgetNode, index) => (
          <MetricWidget
            widget={widgetNode}
            key={`${widgetNode.title}-${index}`}
          />
        ))}
      </SingleValueContainer>
    );
  }

  const valueByKey = useMemo(() => {
    const val = getValueByKey({
      key: value_key,
      keySeparator: key_separator,
      metricUrl: metric_url,
      responseFormat: response_format,
    });
    if (val) {
      return `${val} ${unit || ""}`;
    }
    return fallback_value;
  }, [
    JSON.stringify(globalResponse),
    JSON.stringify(responseDict),
    globalResponseFormat,
  ]);

  return (
    <div
      className={classes.outerContainer}
      onClick={() =>
        widget.description && setFlipped((prevState) => !prevState)
      }
    >
      {flipped ? (
        <div className={classes.descriptionContainer}>
          <p className={classes.description}>{widget.description}</p>
        </div>
      ) : (
        <div className={classes.innerContainer}>
          <p className={classes.label}>{widget.title}</p>

          <p className={classes.value}>{valueByKey}</p>
        </div>
      )}
    </div>
  );
};

export default MetricWidget;
