import { ContainerModule } from "inversify";
import { GalileoIdeMetricsDashboardWidgetWidget } from "./galileo-ide-metrics-dashboard-widget-widget";
import { GalileoIdeMetricsDashboardWidgetContribution } from "./galileo-ide-metrics-dashboard-widget-contribution";
import {
  bindViewContribution,
  createPreferenceProxy,
  FrontendApplicationContribution,
  PreferenceContribution,
  PreferenceProxy,
  PreferenceSchema,
  PreferenceService,
  WidgetFactory,
} from "@theia/core/lib/browser";

import "../../src/browser/style/index.css";
import { EResponseFormat, IMetricWidget } from "./constants";

export const enum GALILEO_METRICS {
  TITLE = "galileo.metrics.title",
  METRIC_URL = "galileo.metrics.metric_url",
  RESPONSE_FORMAT = "galileo.metrics.response_format",
  WIDGETS = "galileo.metrics.widgets",
}

export const galileoMetricsWidgetConfigurationSchema: PreferenceSchema = {
  $schema: "http://json-schema.org/draft-07/schema",
  $id: "http://example.com/example.json",
  type: "object",
  title: "The root schema",
  description: "The root schema comprises the entire JSON document.",
  default: {},
  examples: [
    {
      title: "Metrics Dashboard",
      metric_url: "http://localhost:8080/rpc/metrics",
      response_format: "prometheus",
      widgets: [
        {
          title: "By the numbers",
          type: "single_value_container",
          widgets: [
            {
              title: "Total Incoming Connections",
              value_key: "data/metrics/total_incoming_connections",
              key_separator: "/",
              type: "single_value",
              description_key: "help",
            },
            {
              title: "Avg. Inclusion Distance",
              value_key: "avg_distance",
              type: "single_value",
              description: "Description to explain the metric.",
              unit: "Slot(s)",
            },
            {
              title: "Recent Epoch Gains",
              value_key: "recent_epoch_gains",
              type: "single_value",
              description: "Description to explain the metric.",
              unit: "ETH",
            },
            {
              title: "Correctly Voted Head Percent",
              value_key: "correctly_voted_head",
              type: "single_value",
              description: "Description to explain the metric.",
              unit: "%",
            },
            {
              title: "Validating Keys",
              value_key: "validating_keys",
              type: "single_value",
              description: "Description to explain the metric.",
              metric_url: "http://localhost:8080/rpc/metrics/validating-keys",
              response_format: "json",
            },
          ],
        },
        {
          title: "Chart Name",
          value_keys: ["worst_validator", "avg_validator", "best_validator"],
          description: "Description for chart",
          type: "bar_chart",
          x_axis_label: "X Axis Label",
          y_axis_label: "Y Axis Label",
          legend: [
            {
              key: "worst_validator",
              label: "Worst performing validator",
              color: "#696D84",
            },
            {
              key: "avg_validator",
              label: "Average performing validator",
              color: "#979CB9",
            },
            {
              key: "best_validator",
              label: "Best performing validator",
              color: "#BBC2E5",
            },
          ],
        },
      ],
    },
  ],
  required: ["title", "response_format", "widgets"],
  properties: {
    [GALILEO_METRICS.TITLE]: {
      $id: "#/properties/title",
      type: "string",
      title: "The title schema",
      description: "The title of the Dashboard.",
      default: "",
      examples: ["Metrics Dashboard"],
    },
    [GALILEO_METRICS.METRIC_URL]: {
      $id: "#/properties/metric_url",
      type: "string",
      title: "The metric_url schema",
      description:
        "The URL of the metrics, if this is provided in top level widget's value_key will be searched from this response (if no widget-specific metric_url provided).",
      default: "",
      examples: ["http://localhost:8080/rpc/metrics"],
    },
    [GALILEO_METRICS.RESPONSE_FORMAT]: {
      $id: "#/properties/response_format",
      type: "string",
      title: "The response_format schema",
      description:
        "Could be 'prometheus' or 'json'. According parser will be applied if 'prometheus'.",
      default: "",
      examples: ["prometheus, json"],
    },
    [GALILEO_METRICS.WIDGETS]: {
      $id: "#/properties/widgets",
      type: "array",
      title: "The widgets schema",
      description: "Widget array of the dashboard.",
      default: [],
      examples: [
        [
          {
            title: "By the numbers",
            type: "single_value_container",
            widgets: [
              {
                title: "Total Incoming Connections",
                value_key: "data/metrics/total_incoming_connections",
                key_separator: "/",
                type: "single_value",
                description_key: "help",
              },
              {
                title: "Avg. Inclusion Distance",
                value_key: "avg_distance",
                type: "single_value",
                description: "Description to explain the metric.",
                unit: "Slot(s)",
                fallback_value: "-",
              },
              {
                title: "Recent Epoch Gains",
                value_key: "recent_epoch_gains",
                type: "single_value",
                description: "Description to explain the metric.",
                unit: "ETH",
              },
              {
                title: "Correctly Voted Head Percent",
                value_key: "correctly_voted_head",
                type: "single_value",
                description: "Description to explain the metric.",
                unit: "%",
              },
              {
                title: "Validating Keys",
                value_key: "validating_keys",
                type: "single_value",
                description: "Description to explain the metric.",
                metric_url: "http://localhost:8080/rpc/metrics/validating-keys",
                response_format: "json",
              },
            ],
          },
          {
            title: "Chart Name",
            value_keys: ["worst_validator", "avg_validator", "best_validator"],
            description: "Description for chart",
            type: "bar_chart",
            x_axis_label: "X Axis Label",
            y_axis_label: "Y Axis Label",
            legend: [
              {
                key: "worst_validator",
                label: "Worst performing validator",
                color: "#696D84",
              },
              {
                key: "avg_validator",
                label: "Average performing validator",
                color: "#979CB9",
              },
              {
                key: "best_validator",
                label: "Best performing validator",
                color: "#BBC2E5",
              },
            ],
          },
        ],
      ],
      additionalItems: true,
      items: {
        $id: "#/properties/widgets/items",
        anyOf: [
          {
            $id: "#/properties/widgets/items/anyOf/0",
            type: "object",
            title: "The first anyOf schema",
            description: "An explanation about the purpose of this instance.",
            default: {},
            examples: [
              {
                title: "By the numbers",
                type: "single_value_container",
                widgets: [
                  {
                    title: "Total Incoming Connections",
                    value_key: "data/metrics/total_incoming_connections",
                    key_separator: "/",
                    type: "single_value",
                    description_key: "help",
                  },
                  {
                    title: "Avg. Inclusion Distance",
                    value_key: "avg_distance",
                    type: "single_value",
                    description: "Description to explain the metric.",
                    unit: "Slot(s)",
                    fallback_value: "-",
                  },
                  {
                    title: "Recent Epoch Gains",
                    value_key: "recent_epoch_gains",
                    type: "single_value",
                    description: "Description to explain the metric.",
                    unit: "ETH",
                  },
                  {
                    title: "Correctly Voted Head Percent",
                    value_key: "correctly_voted_head",
                    type: "single_value",
                    description: "Description to explain the metric.",
                    unit: "%",
                  },
                  {
                    title: "Validating Keys",
                    value_key: "validating_keys",
                    type: "single_value",
                    description: "Description to explain the metric.",
                    metric_url:
                      "http://localhost:8080/rpc/metrics/validating-keys",
                    response_format: "json",
                  },
                ],
              },
            ],
            required: ["title", "type", "widgets"],
            properties: {
              title: {
                $id: "#/properties/widgets/items/anyOf/0/properties/title",
                type: "string",
                title: "The title schema",
                description:
                  "An explanation about the purpose of this instance.",
                default: "",
                examples: ["By the numbers"],
              },
              type: {
                $id: "#/properties/widgets/items/anyOf/0/properties/type",
                type: "string",
                title: "The type schema",
                description:
                  "An explanation about the purpose of this instance.",
                default: "",
                examples: ["single_value_container"],
              },
              widgets: {
                $id: "#/properties/widgets/items/anyOf/0/properties/widgets",
                type: "array",
                title: "The widgets schema",
                description:
                  "An explanation about the purpose of this instance.",
                default: [],
                examples: [
                  [
                    {
                      title: "Total Incoming Connections",
                      value_key: "data/metrics/total_incoming_connections",
                      key_separator: "/",
                      type: "single_value",
                      description_key: "help",
                    },
                    {
                      title: "Avg. Inclusion Distance",
                      value_key: "avg_distance",
                      type: "single_value",
                      description: "Description to explain the metric.",
                      unit: "Slot(s)",
                    },
                  ],
                ],
                additionalItems: true,
                items: {
                  $id: "#/properties/widgets/items/anyOf/0/properties/widgets/items",
                  anyOf: [
                    {
                      $id: "#/properties/widgets/items/anyOf/0/properties/widgets/items/anyOf/0",
                      type: "object",
                      title: "The first anyOf schema",
                      description:
                        "An explanation about the purpose of this instance.",
                      default: {},
                      examples: [
                        {
                          title: "Total Incoming Connections",
                          value_key: "total_incoming_connections",
                          type: "single_value",
                          description_key: "help",
                        },
                      ],
                      required: [
                        "title",
                        "value_key",
                        "type",
                        "description_key",
                      ],
                      properties: {
                        title: {
                          $id: "#/properties/widgets/items/anyOf/0/properties/widgets/items/anyOf/0/properties/title",
                          type: "string",
                          title: "The title schema",
                          description:
                            "An explanation about the purpose of this instance.",
                          default: "",
                          examples: ["Total Incoming Connections"],
                        },
                        value_key: {
                          $id: "#/properties/widgets/items/anyOf/0/properties/widgets/items/anyOf/0/properties/value_key",
                          type: "string",
                          title: "The value_key schema",
                          description:
                            "An explanation about the purpose of this instance.",
                          default: "",
                          examples: ["data/metrics/total_incoming_connections"],
                        },
                        key_separator: {
                          $id: "#/properties/widgets/items/anyOf/0/properties/widgets/items/anyOf/0/properties/key_separator",
                          type: "string",
                          title: "The key_separator schema",
                          description:
                            "An explanation about the purpose of this instance.",
                          default: "",
                          examples: ["/"],
                        },
                        fallback_value: {
                          $id: "#/properties/widgets/items/anyOf/0/properties/widgets/items/anyOf/0/properties/fallback_value",
                          type: "string",
                          title: "The fallback_value schema",
                          description:
                            "An explanation about the purpose of this instance.",
                          default: "",
                          examples: ["-"],
                        },
                        type: {
                          $id: "#/properties/widgets/items/anyOf/0/properties/widgets/items/anyOf/0/properties/type",
                          type: "string",
                          title: "The type schema",
                          description:
                            "An explanation about the purpose of this instance.",
                          default: "",
                          examples: ["single_value"],
                        },
                        description_key: {
                          $id: "#/properties/widgets/items/anyOf/0/properties/widgets/items/anyOf/0/properties/description_key",
                          type: "string",
                          title: "The description_key schema",
                          description:
                            "An explanation about the purpose of this instance.",
                          default: "",
                          examples: ["help"],
                        },
                      },
                      additionalProperties: true,
                    },
                    {
                      $id: "#/properties/widgets/items/anyOf/0/properties/widgets/items/anyOf/1",
                      type: "object",
                      title: "The second anyOf schema",
                      description:
                        "An explanation about the purpose of this instance.",
                      default: {},
                      examples: [
                        {
                          title: "Avg. Inclusion Distance",
                          value_key: "avg_distance",
                          type: "single_value",
                          description: "Description to explain the metric.",
                          unit: "Slot(s)",
                        },
                      ],
                      required: [
                        "title",
                        "value_key",
                        "type",
                        "description",
                        "unit",
                      ],
                      properties: {
                        title: {
                          $id: "#/properties/widgets/items/anyOf/0/properties/widgets/items/anyOf/1/properties/title",
                          type: "string",
                          title: "The title schema",
                          description:
                            "An explanation about the purpose of this instance.",
                          default: "",
                          examples: ["Avg. Inclusion Distance"],
                        },
                        value_key: {
                          $id: "#/properties/widgets/items/anyOf/0/properties/widgets/items/anyOf/1/properties/value_key",
                          type: "string",
                          title: "The value_key schema",
                          description:
                            "An explanation about the purpose of this instance.",
                          default: "",
                          examples: ["avg_distance"],
                        },
                        type: {
                          $id: "#/properties/widgets/items/anyOf/0/properties/widgets/items/anyOf/1/properties/type",
                          type: "string",
                          title: "The type schema",
                          description:
                            "An explanation about the purpose of this instance.",
                          default: "",
                          examples: ["single_value"],
                        },
                        description: {
                          $id: "#/properties/widgets/items/anyOf/0/properties/widgets/items/anyOf/1/properties/description",
                          type: "string",
                          title: "The description schema",
                          description:
                            "An explanation about the purpose of this instance.",
                          default: "",
                          examples: ["Description to explain the metric."],
                        },
                        unit: {
                          $id: "#/properties/widgets/items/anyOf/0/properties/widgets/items/anyOf/1/properties/unit",
                          type: "string",
                          title: "The unit schema",
                          description:
                            "An explanation about the purpose of this instance.",
                          default: "",
                          examples: ["Slot(s)"],
                        },
                      },
                      additionalProperties: true,
                    },
                    {
                      $id: "#/properties/widgets/items/anyOf/0/properties/widgets/items/anyOf/2",
                      type: "object",
                      title: "The third anyOf schema",
                      description:
                        "An explanation about the purpose of this instance.",
                      default: {},
                      examples: [
                        {
                          title: "Validating Keys",
                          value_key: "validating_keys",
                          type: "single_value",
                          description: "Description to explain the metric.",
                          metric_url:
                            "http://localhost:8080/rpc/metrics/validating-keys",
                          response_format: "json",
                        },
                      ],
                      required: [
                        "title",
                        "value_key",
                        "type",
                        "description",
                        "metric_url",
                        "response_format",
                      ],
                      properties: {
                        title: {
                          $id: "#/properties/widgets/items/anyOf/0/properties/widgets/items/anyOf/2/properties/title",
                          type: "string",
                          title: "The title schema",
                          description:
                            "An explanation about the purpose of this instance.",
                          default: "",
                          examples: ["Validating Keys"],
                        },
                        value_key: {
                          $id: "#/properties/widgets/items/anyOf/0/properties/widgets/items/anyOf/2/properties/value_key",
                          type: "string",
                          title: "The value_key schema",
                          description:
                            "An explanation about the purpose of this instance.",
                          default: "",
                          examples: ["validating_keys"],
                        },
                        type: {
                          $id: "#/properties/widgets/items/anyOf/0/properties/widgets/items/anyOf/2/properties/type",
                          type: "string",
                          title: "The type schema",
                          description:
                            "An explanation about the purpose of this instance.",
                          default: "",
                          examples: ["single_value"],
                        },
                        description: {
                          $id: "#/properties/widgets/items/anyOf/0/properties/widgets/items/anyOf/2/properties/description",
                          type: "string",
                          title: "The description schema",
                          description:
                            "An explanation about the purpose of this instance.",
                          default: "",
                          examples: ["Description to explain the metric."],
                        },
                        metric_url: {
                          $id: "#/properties/widgets/items/anyOf/0/properties/widgets/items/anyOf/2/properties/metric_url",
                          type: "string",
                          title: "The metric_url schema",
                          description:
                            "An explanation about the purpose of this instance.",
                          default: "",
                          examples: [
                            "http://localhost:8080/rpc/metrics/validating-keys",
                          ],
                        },
                        response_format: {
                          $id: "#/properties/widgets/items/anyOf/0/properties/widgets/items/anyOf/2/properties/response_format",
                          type: "string",
                          title: "The response_format schema",
                          description:
                            "An explanation about the purpose of this instance.",
                          default: "",
                          examples: ["json"],
                        },
                      },
                      additionalProperties: true,
                    },
                  ],
                },
              },
            },
            additionalProperties: true,
          },
          {
            $id: "#/properties/widgets/items/anyOf/1",
            type: "object",
            title: "The second anyOf schema",
            description: "An explanation about the purpose of this instance.",
            default: {},
            examples: [
              {
                title: "Chart Name",
                value_keys: [
                  "worst_validator",
                  "avg_validator",
                  "best_validator",
                ],
                description: "Description for chart",
                type: "bar_chart",
                x_axis_label: "X Axis Label",
                y_axis_label: "Y Axis Label",
                legend: [
                  {
                    key: "worst_validator",
                    label: "Worst performing validator",
                    color: "#696D84",
                  },
                  {
                    key: "avg_validator",
                    label: "Average performing validator",
                    color: "#979CB9",
                  },
                  {
                    key: "best_validator",
                    label: "Best performing validator",
                    color: "#BBC2E5",
                  },
                ],
              },
            ],
            required: [
              "title",
              "value_keys",
              "description",
              "type",
              "x_axis_label",
              "y_axis_label",
              "legend",
            ],
            properties: {
              title: {
                $id: "#/properties/widgets/items/anyOf/1/properties/title",
                type: "string",
                title: "The title schema",
                description:
                  "An explanation about the purpose of this instance.",
                default: "",
                examples: ["Chart Name"],
              },
              value_keys: {
                $id: "#/properties/widgets/items/anyOf/1/properties/value_keys",
                type: "array",
                title: "The value_keys schema",
                description:
                  "An explanation about the purpose of this instance.",
                default: [],
                examples: [["worst_validator", "avg_validator"]],
                additionalItems: true,
                items: {
                  $id: "#/properties/widgets/items/anyOf/1/properties/value_keys/items",
                  anyOf: [
                    {
                      $id: "#/properties/widgets/items/anyOf/1/properties/value_keys/items/anyOf/0",
                      type: "string",
                      title: "The first anyOf schema",
                      description:
                        "An explanation about the purpose of this instance.",
                      default: "",
                      examples: ["worst_validator", "avg_validator"],
                    },
                  ],
                },
              },
              description: {
                $id: "#/properties/widgets/items/anyOf/1/properties/description",
                type: "string",
                title: "The description schema",
                description:
                  "An explanation about the purpose of this instance.",
                default: "",
                examples: ["Description for chart"],
              },
              type: {
                $id: "#/properties/widgets/items/anyOf/1/properties/type",
                type: "string",
                title: "The type schema",
                description:
                  "An explanation about the purpose of this instance.",
                default: "",
                examples: ["bar_chart"],
              },
              x_axis_label: {
                $id: "#/properties/widgets/items/anyOf/1/properties/x_axis_label",
                type: "string",
                title: "The x_axis_label schema",
                description:
                  "An explanation about the purpose of this instance.",
                default: "",
                examples: ["X Axis Label"],
              },
              y_axis_label: {
                $id: "#/properties/widgets/items/anyOf/1/properties/y_axis_label",
                type: "string",
                title: "The y_axis_label schema",
                description:
                  "An explanation about the purpose of this instance.",
                default: "",
                examples: ["Y Axis Label"],
              },
              legend: {
                $id: "#/properties/widgets/items/anyOf/1/properties/legend",
                type: "array",
                title: "The legend schema",
                description:
                  "An explanation about the purpose of this instance.",
                default: [],
                examples: [
                  [
                    {
                      key: "worst_validator",
                      label: "Worst performing validator",
                      color: "#696D84",
                    },
                    {
                      key: "avg_validator",
                      label: "Average performing validator",
                      color: "#979CB9",
                    },
                  ],
                ],
                additionalItems: true,
                items: {
                  $id: "#/properties/widgets/items/anyOf/1/properties/legend/items",
                  anyOf: [
                    {
                      $id: "#/properties/widgets/items/anyOf/1/properties/legend/items/anyOf/0",
                      type: "object",
                      title: "The first anyOf schema",
                      description:
                        "An explanation about the purpose of this instance.",
                      default: {},
                      examples: [
                        {
                          key: "worst_validator",
                          label: "Worst performing validator",
                          color: "#696D84",
                        },
                      ],
                      required: ["key", "label", "color"],
                      properties: {
                        key: {
                          $id: "#/properties/widgets/items/anyOf/1/properties/legend/items/anyOf/0/properties/key",
                          type: "string",
                          title: "The key schema",
                          description:
                            "An explanation about the purpose of this instance.",
                          default: "",
                          examples: ["worst_validator"],
                        },
                        label: {
                          $id: "#/properties/widgets/items/anyOf/1/properties/legend/items/anyOf/0/properties/label",
                          type: "string",
                          title: "The label schema",
                          description:
                            "An explanation about the purpose of this instance.",
                          default: "",
                          examples: ["Worst performing validator"],
                        },
                        color: {
                          $id: "#/properties/widgets/items/anyOf/1/properties/legend/items/anyOf/0/properties/color",
                          type: "string",
                          title: "The color schema",
                          description:
                            "An explanation about the purpose of this instance.",
                          default: "",
                          examples: ["#696D84"],
                        },
                      },
                      additionalProperties: true,
                    },
                  ],
                },
              },
            },
            additionalProperties: true,
          },
        ],
      },
    },
  },
  additionalProperties: true,
};

export interface galileoMetricsWidgetConfiguration {
  "galileo.metrics.title": string;
  "galileo.metrics.metric_url": string;
  "galileo.metrics.response_format": EResponseFormat;
  "galileo.metrics.widgets": IMetricWidget[];
}

export const GalileoMetricsWidgetPreferences = Symbol(
  "GalileoMetricsWidgetPreferences"
);
export type GalileoMetricsWidgetPreferences =
  PreferenceProxy<galileoMetricsWidgetConfiguration>;

export function createGalileoMetricsWidgetPreferences(
  preferences: PreferenceService
): GalileoMetricsWidgetPreferences {
  return createPreferenceProxy(
    preferences,
    galileoMetricsWidgetConfigurationSchema
  );
}
export default new ContainerModule((bind) => {
  bindViewContribution(bind, GalileoIdeMetricsDashboardWidgetContribution);
  bind(FrontendApplicationContribution).toService(
    GalileoIdeMetricsDashboardWidgetContribution
  );
  bind(GalileoIdeMetricsDashboardWidgetWidget).toSelf();
  bind(WidgetFactory)
    .toDynamicValue((ctx) => ({
      id: GalileoIdeMetricsDashboardWidgetWidget.ID,
      createWidget: () =>
        ctx.container.get<GalileoIdeMetricsDashboardWidgetWidget>(
          GalileoIdeMetricsDashboardWidgetWidget
        ),
    }))
    .inSingletonScope();
  bind(GalileoMetricsWidgetPreferences).toDynamicValue((ctx) => {
    const preferences = ctx.container.get<PreferenceService>(PreferenceService);
    return createGalileoMetricsWidgetPreferences(preferences);
  });
  bind(PreferenceContribution).toConstantValue({
    schema: galileoMetricsWidgetConfigurationSchema,
  });
});
